@@ -1,6 +1,6 @@
 ;;;; chicken-install.scm
 ;
-; Copyright (c) 2008-2016, The CHICKEN Team
+; Copyright (c) 2008-2017, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -97,6 +97,7 @@
   (define *csc-features* '())
   (define *csc-nonfeatures* '())
   (define *prefix* #f)
+  (define *no-install-deps* #f)
   (define *aliases* '())
   (define *cross-chicken* (feature? #:cross-chicken))
   (define *host-extension* *cross-chicken*)
@@ -467,7 +468,9 @@
 		      (check-platform (car e+d+v) meta)
                       (print "checking dependencies for `" (car e+d+v) "' ...")
                       (let-values (((missing upgrade) 
-				    (outdated-dependencies (car e+d+v) meta)))
+				    (if *no-install-deps*
+				      (values '() '())
+				      (outdated-dependencies (car e+d+v) meta))))
 			(set! missing (apply-mappings missing)) ;XXX only missing - wrong?
 			(set! *dependencies*
 			  (cons
@@ -582,7 +585,7 @@
     (when *keep-existing*
       (set! eggs
 	(remove 
-	 (lambda (egg) (extension-information (if (pair? egg) (car egg) egg)))
+	 (lambda (egg) (ext-version (if (pair? egg) (car egg) egg)))
 	 eggs)))
     (retrieve eggs)
     (unless *retrieve-only*
@@ -814,6 +817,7 @@ usage: chicken-install [OPTION | EXTENSION[:VERSION]] ...
   -r   -retrieve                only retrieve egg into current directory, don't install
   -n   -no-install              do not install, just build (implies `-keep')
   -p   -prefix PREFIX           change installation prefix to PREFIX
+       -no-install-deps         do not install dependencies
        -list                    list extensions available over selected transport and location
        -host                    when cross-compiling, compile extension only for host
        -target                  when cross-compiling, compile extension only for target
@@ -953,6 +957,9 @@ EOF
 				(normalize-pathname 
 				 (make-pathname (current-directory) p) ) ) ) )
                         (loop (cddr args) eggs))
+		       ((string=? arg "-no-install-deps")
+			(set! *no-install-deps* #t)
+			(loop (cdr args) eggs))
                        ((or (string=? arg "-n") (string=? arg "-no-install"))
                         (set! *keep* #t)
                         (set! *no-install* #t)
