@@ -163,13 +163,10 @@
     (define (constant-result lit)
       (cond ((string? lit) 'string)
 	    ((symbol? lit) 'symbol)
+	    ;; Do not assume fixnum width matches target platforms!
+	    ((big-fixnum? lit) 'number)
 	    ((fixnum? lit) 'fixnum)
 	    ((flonum? lit) 'float)
-	    ((number? lit) 
-	     (case number-type 
-	       ((fixnum) 'fixnum)
-	       ((flonum) 'flonum)
-	       (else 'number)))		; in case...
 	    ((boolean? lit)
 	     (if lit 'true 'false))
 	    ((null? lit) 'null)
@@ -494,6 +491,10 @@
 		 ((##core#proc) '(procedure))
 		 ((##core#global-ref) (global-result (first params) loc))
 		 ((##core#variable) (variable-result (first params) e loc flow))
+		 ((##core#inline_ref)
+		  (list (foreign-type->scrutiny-type (second params) 'result)))
+		 ((##core#inline_loc_ref)
+		  (list (foreign-type->scrutiny-type (first  params) 'result)))
 		 ((if)
 		  (let ((tags (cons (tag) (tag)))
 			(tst (first subs))
@@ -696,7 +697,7 @@
 		      (set-cdr! params '(#t)))
 
 		    '(undefined)))
-		 ((##core#primitive ##core#inline_ref) '*)
+		 ((##core#primitive) '*)
 		 ((##core#call)
 		  (let* ((f (fragment n))
 			 (len (length subs))
@@ -1219,13 +1220,12 @@
 			       constraints))
 		     (simplify (third t))))
 		  ((or)
- 		   (let* ((ts (map simplify (cdr t)))
- 			  (tslen (length ts)))
- 		     (cond ((= 1 tslen) (car ts))
-			   ((null? ts) '*)
- 			   ((> tslen +maximal-union-type-length+)
- 			    (d "union-type cutoff! (~a): ~s" tslen ts)
- 			    '*)
+		   (let ((ts (delete-duplicates (map simplify (cdr t)) eq?)))
+		     (cond ((null? ts) '*)
+			   ((null? (cdr ts)) (car ts))
+			   ((> (length ts) +maximal-union-type-length+)
+			    (d "union-type cutoff! (~a): ~s" (length ts) ts)
+			    '*)
 			   ((every procedure-type? ts)
 			    (if (any (cut eq? 'procedure <>) ts)
 				'procedure
@@ -1253,7 +1253,7 @@
 					    (cond ((and (pair? t) (eq? 'or (car t)))
 						   (cdr t))
 						  ((eq? t 'undefined) (return 'undefined))
-						  ((eq? t 'noreturn) '())
+						  ((eq? t 'noreturn) (return '*))
 						  (else (list t)))))
 					ts))
 				   (ts2 (let loop ((ts ts) (done '()))
@@ -1355,11 +1355,8 @@
   (call/cc
    (lambda (return)
      (let loop ((ts1 ts11) (ts2 ts21))
-       (cond ((null? ts1) '())
-	     ((null? ts2) '())
+       (cond ((and (null? ts1) (null? ts2)) '())
 	     ((or (atom? ts1) (atom? ts2)) (return '*))
-	     ((eq? 'noreturn (car ts1)) (loop (cdr ts1) ts2))
-	     ((eq? 'noreturn (car ts2)) (loop ts1 (cdr ts2)))
 	     (else (cons (simplify-type `(or ,(car ts1) ,(car ts2)))
 			 (loop (cdr ts1) (cdr ts2)))))))))
 
